#!/bin/bash

# Copy the data from one database to another

set -e

# Create file like this:
#
# SOURCE_HOST=mysql-app.stanford.edu
# SOURCE_DB=s_spdb_prod
# SOURCE_USER=s_spdb_prod
# SOURCE_PW=secret
# DEST_HOST=mysql-app.stanford.edu
# DEST_DB=s_spdb_dev
# DEST_USER=s_spdb_dev
# DEST_PW=secret

# Then run this script with a single argument of the name of the above
# created file.

if [[ "$1" == "" ]]; then
    echo "error: missing database configuration file name"
    exit 1
fi

source $1

# It is VERY important we don't overwrite production.
if [[ "$DEST_DB" =~ prod ]]; then
    print "You are trying to overwrite production? NO!!!"
    exit 1
fi

SOURCE_SQL=/tmp/source.sql

# Dump source
rm -f $SOURCE_SQL
echo "dumping database '$SOURCE_DB'..."
mysqldump -u "$SOURCE_USER" -p"$SOURCE_PW" -h "$SOURCE_HOST" "$SOURCE_DB" > "$SOURCE_SQL"

# Load into destination
echo "loading into destination database '$DEST_DB'..."
mysql -u "$DEST_USER" -p"$DEST_PW" -h "$DEST_HOST" "$DEST_DB" < "$SOURCE_SQL"

# Clean up
echo "cleaning up..."
rm -f "$SOURCE_SQL"

echo "be sure to remove $1 (it has passwords!!)"
