class ApplicationController < ActionController::Base
  require 'utility'

  before_action :set_start_time

  # Prevent CSRF attacks by raising an exception.
  # For APIs, you may want to use :null_session instead.
  protect_from_forgery with: :exception

  # act_as_admin returns true if the current user is in the admin list AND
  # the act_as_admin session variable is true. Otherwise, returns false.
  def act_as_admin()
    if (session[:act_as_admin] and Utility.is_admin(get_REMOTE_USER())) then
      return true
    else
      return false
    end
  end

  protected
    # All these methods are available to all controllers.

    # Use request.env rather than ENV. ENV seems to get values from other
    # Apache processes confusing the application about who is really logged
    # in.
    def set_remote_user
      $remote_user = request.env['REMOTE_USER']

      # Die if $remote_user is not set.
      if ($remote_user.blank?()) then
        raise "cannot continue as REMOTE_USER is not set"
      end

      Utility.logger.info("REMOTE_USER is #{$remote_user}")

      # Also set the applications this user can see.
      $visible_applications = Utility.visible_applications($remote_user)
      Utility.logger.info("$visible_applications is #{$visible_applications.inspect()}")
    end

    # This is used by paper_trail. Make sure to call this AFTER set_remote_user
    def current_user
      return $remote_user
    end

    def set_is_admin
      @is_admin          = false
      @is_readonly_admin = false
      remote_user        = Utility.get_REMOTE_USER()

      if (Utility.is_admin(remote_user)) then
        @is_admin = true
      elsif (Utility.is_readonly_admin(remote_user)) then
        @is_readonly_admin = true
      end
    end

    def set_start_time
      @start_time = Time.now.to_f
    end

end
