class Certificate < ActiveRecord::Base
  belongs_to :spconfig

  # Get the CN
  def cn
    return Utility.subject_to_cn(self.subject)
  end

  # Returns the date (not time) of expiration
  def expires_date
    return self.expires_at.strftime("%m/%d/%Y")
  end

  # Returns one of:
  #  'expired'
  #  'warning' (expires is less than two years)
  #  'good' (expires in two years or more)
  def expiration_level
    curtime  = DateTime.now.strftime('%s').to_i

    if (Time.at(self.expires_at).to_i > curtime)
      if (curtime > Time.at(self.expires_at).to_i - 63072000)
        return 'warning'
      else
        return 'good'
      end
    else
      return 'expired'
    end
  end

  def is_expired?()
    return self.expiration_level() == 'expired'
  end

  def is_almost_expired?()
    return self.expiration_level() == 'warning'
  end

  def use_is_encryption?()
    if (self.use == 'encryption')
      return true
    else
      return false
    end
  end

  def use_is_signing?()
    if (self.use == 'signing')
      return true
    else
      return false
    end
  end

  def use_is_unspecified?()
    if (self.use == 'unspecified')
      return true
    else
      return false
    end
  end

  def can_be_used_for_encryption?()
    return (self.use_is_encryption() or self.use_is_unspecified())
  end

end
