# Export combined metadata file.
require 'utility'

# Spconfig.save_metadata_to_file()
$metadata_type_to_was_written = Spconfig.save_metadatas_to_file()

# Make sure the permissions are correct
basepath = Rails.configuration.sp_metadatafile_basedir

cmd = "chown -R root.www-data #{basepath}"
`#{cmd}`

cmd = "chmod -R g+rw #{basepath}"
`#{cmd}`


def send_to_cloud_storage(cloud_provider, bucket_name, credentials_file)
  provider_to_script = {
    'aws' => '/usr/share/spdb/bin/send-metadata-to-s3',
    'gcp' => '/usr/share/spdb/bin/send-metadata-to-gcs',
  }

  if (not provider_to_script.has_key?(cloud_provider)) then
    Utility.logger.error("unrecognized cloud provider '#{cloud_provider}'")
    exit(1)
  end


  if ($metadata_type_to_was_written['combined']) then
    Utility.logger.info("[export_metadata] sending combined metadata to #{cloud_provider}")

    directory = Rails.env

    cmds = [provider_to_script[cloud_provider],
            bucket_name,
            directory,
            "--credentials-file=#{credentials_file}",
           ]

    cmd = cmds.join(' ')
    Utility.logger.debug("[export_metadata] about to run command '#{cmd}'")

    stdout, stderr, p_status = Open3.capture3(cmd)
    rc = p_status.exitstatus()
    Utility.logger.debug("[#{stdout}][#{stderr}][#{p_status}][#{rc}]")

    if (rc == 0) then
      Utility.logger.debug("#{cloud_provider} upload command suceeded")
    else
      # An error.
      Utility.logger.error("#{cloud_provider} command '#{cmd}' failed: #{stderr}")
      exit(1)
    end
  else
    Utility.logger.info("[export_metadata] not time to send combined metadata to #{cloud_provider}")
  end
end

# If the 'combined' file was written, send it to Amazon S3 (maybe) and
# Google Cloud Storage (maybe).

# AWS
if (Rails.configuration.combined_metadata_to_s3_send) then
    bucket_name = Rails.configuration.combined_metadata_to_s3_bucket
    credentials = Rails.configuration.combined_metadata_to_s3_credentials
    send_to_cloud_storage('aws', bucket_name, credentials)
else
  Utility.logger.info("[export_metadata] not sending metadata to S3 (send disabled)")
end

# GCP
if (Rails.configuration.combined_metadata_to_gcs_send) then
    bucket_name = Rails.configuration.combined_metadata_to_gcs_bucket
    credentials = Rails.configuration.combined_metadata_to_gcs_credentials
    send_to_cloud_storage('gcp', bucket_name, credentials)
else
  Utility.logger.info("[export_metadata] not sending metadata to GCP (send disabled)")
end
