#!/usr/bin/python
import traceback
from time import sleep
from xml.sax.saxutils import escape, quoteattr
import os, sys, unittest, time, re, random, string, tempfile
from datetime import datetime
from datetime import timedelta
from subprocess import call
import argparse
import inspect
import tempfile
import pprint

# In case selenium module is not installed.
try:
  from selenium import webdriver
  from selenium.webdriver.chrome.options import Options
  from selenium.webdriver.common.by import By
  from selenium.webdriver.common.keys import Keys
  from selenium.webdriver.common.action_chains import ActionChains
  from selenium.webdriver.support.ui import Select
  from selenium.webdriver.support.ui import WebDriverWait
  from selenium.webdriver.support import expected_conditions as EC
  from selenium.common.exceptions import UnexpectedAlertPresentException
  from selenium.common.exceptions import StaleElementReferenceException
  from selenium.common.exceptions import NoAlertPresentException
  from selenium.common.exceptions import NoSuchElementException
  from selenium.webdriver.common.alert import Alert
except ImportError:
  pass

# Note: Remember that WebDriverWait(driver, 30).until(some_method)
# repeatedly calls "some_method(driver)" until that call returns true, or
# else the time runs out.

# If the document is in the "complete" state (i.e., the document has
# completed loading) return true, otherwise return false.
def readystate_complete(driver):
  return driver.execute_script("return document.readyState") == "complete"

class SPDBTesting(object):

  WG1 = 'as-fss:tech'
  WG2 = 'as-fss:dmrteam'

  # Set up the webdriver object. To access it, use "self.driver".
  def __init__(self, environment, browser, username = 'unknown', verbose = False):

    self.environment = environment.lower()
    self.browser     = browser
    self.username    = username
    self.password    = 'spdb-quality-assurance'
    self.awsaccount  = '012345678999'

    self.verbose     = verbose
    self.pprint      = pprint.PrettyPrinter(indent=4)

    if (browser == 'chrome'):
      if (sys.platform.lower() == 'win32'):
        chromedriver = "D:\\bin\\chromedriver.exe"
        os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome(chromedriver)
      else:
        chromedriver = "/usr/sbin/chromedriver"
        chrome_options = Options()
        chrome_options.add_argument("--disable-infobars")
        os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome(chrome_options=chrome_options)
    elif (browser == 'firefox'):
      self.driver = webdriver.Firefox()
    elif (browser == 'phantomjs'):
      self.driver = webdriver.PhantomJS(service_args=['--ignore-ssl-errors=true'])
      self.driver.set_window_size(1120, 550)
    else:
      raise ValueError("unknown browser " + browser)

    self.fake_wait   = True
    self.test_string = None
    self.driver.maximize_window()
    self.driver.implicitly_wait(30)
    self.use_basic_auth = True

    self.hostname = 'spdb-' + environment + '.stanford.edu'

    # Set the base url
    self.base_url = None
    self.set_base_url()

    self.verificationErrors = []
    self.accept_next_alert = True
    print self.base_url


  def progress(self, message):
    if (self.verbose):
      if (message is None):
        message = 'None'
      print('progress: ' + message)

  def pp(self, object):
    return self.pprint.pformat(object)

  def exit_with_exception(self, message):
    page_source = self.driver.page_source.encode('utf-8').strip()

    fp = open('out.html', 'w')
    fp.write(page_source)
    fp.close()

    self.driver.save_screenshot('out.png');

    self.driver.quit();
    raise Exception(message + "; see screen shot in out.png and page source is in out.html")


  # When running a workgroup membership test we need to have
  # a real user (e.g., "adamhl", "cheroux", "vivienwu", etc.). Note
  # that when we set the username the url changes as well.
  def reset_username(self, user_name):
    self.username = user_name
    self.set_base_url()

  # Set the url attribute. This will depend on the authentication
  # mechanism and the user
  def set_base_url(self):
    if (self.use_basic_auth):
      url = "https://{}:{}@{}/awsaccounts".format(self.username, self.password, self.hostname)
    else:
      url = "https://{}/awsaccounts".format(self.hostname)

    self.base_url = url
    return self.base_url

  def go_home(self):
    return self.driver.get(self.base_url)

  def go_to_menu_option(self, button_id):
    driver = self.driver
    self.go_home()
    #
    # if (button_id == "/awsaccounts/all"):
    #   driver.get(button_id)
    # else:
    #
    driver.find_element_by_id(button_id).click()

    WebDriverWait(driver, 30).until(readystate_complete)
    return driver.page_source

  # Get the HTML that comprises the red error text (flash) box at the top
  # of the screen. If there is no such box return None.
  def get_red_error_text(self):
    box_id = 'error_explanation'
    error_box_element = None
    try:
      error_box_element = self.driver.find_element_by_id(box_id)
    except NoSuchElementException, e:
      # No such element, so return None.
      return None

    return error_box_element.get_attribute('innerHTML')

  # Get the HTML that comprises the green text (flash) box at the top
  # of the screen. If there is no such box return None.
  def get_message_text(self):
    box_id = 'notice_box'
    message_box_element = None
    try:
      message_box_element = self.driver.find_element_by_class_name(box_id)
    except NoSuchElementException, e:
      # No such element, so return None.
      return None

    return message_box_element.get_attribute('innerHTML')

  # Use send_keys on an element but only chunk_size characters at a time.
  def send_keys_slow(self, element, text, chunk_size=128):
    while (len(text) > 0):
      current_chunk = text[0:chunk_size]
      element.send_keys(current_chunk)
      text = text[chunk_size:]

  def get_element_by_id(self, id):
    return self.driver.find_element_by_id(id)

  # Go to the "Add New Service Provider" page.
  def go_to_add_new(self):
    driver = self.driver
    self.go_home()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("button_spconfig_new").click()
    WebDriverWait(driver, 30).until(readystate_complete)

  # Submits the metadata but does not return anything.
  def test_metadata(self, metadata_string):
    self.go_to_add_new()

    driver = self.driver
    time.sleep(5)

    # We require at least one workgroup
    self.add_workgroup_on_edit_form(self.WG1 + '|0')

    driver.find_element_by_id("spconfig_metadata").clear()
    element = driver.find_element_by_id("spconfig_metadata")
    self.send_keys_slow(element, metadata_string)

    driver.find_element_by_id("spconfig_contact_email").clear()
    driver.find_element_by_id("spconfig_contact_email").send_keys("as-acs-linux-teaam@lists.stanford.edu")

    driver.find_element_by_link_text("Submit Metadata").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    if (self.fake_wait):
      time.sleep(10)

    return

  #
  # Use the search function to find one or more SP configs.
  #
  def search_for_aws(self, search_text):
    driver = self.driver

    self.go_home()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("search").clear()
    driver.find_element_by_id("search").send_keys(search_text)
    driver.find_element_by_id("button_spconfig_search").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    if ("Search found 0 AWS" in driver.page_source):
      return 0
    else:
      return 1

  #
  # Assuming we are on the view page and there is only ONE SP configuration listed,
  # click the update icon.
  #
  def click_update_icon(self):
    try:
      update_link = self.driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]")
    except:
      self.exit_with_exception("could not find update icon using css selector")

    update_link.click()

  # If we are on the edit form, add the supplied workgroup
  # Expects workgroup to be a string of the form 'workgroup_name|#' where
  # # is an integer.
  def add_workgroup_on_edit_form(self, workgroup):
    temp_name      = workgroup.split("|")
    workgroup_name = temp_name[0]
    workgroup_id   = int(temp_name[1])

    driver = self.driver
    driver.find_element_by_css_selector("img[alt=\"Add workgroup\"]").click()

    element_id     = None
    input_elements = driver.find_elements_by_xpath("//*[contains(@id, 'awsaccount_awsowninggroups_')]")
    self.progress('add_workgroup input_elements')
    self.progress(self.pp(input_elements))
    if (not input_elements):
      self.exit_with_exception("could not find add workgroup link")

    for element in input_elements:
      element_name = element.get_attribute('name')
      if ("workgroup" in element_name):
        element_id = element.get_attribute('id')
        if (workgroup_id == 0):
          break

    if (element_id is None):
      self.exit_with_exception("could not find add workgroup element")

    driver.find_element_by_id(element_id).clear()
    driver.find_element_by_id(element_id).send_keys(workgroup_name)
    time.sleep(2);

    return

  # Add a workgroup to the SP configuration with the provided
  # entityID. Returns the contents of the owning workgroup element (or
  # None if there is no such element).
  def add_workgroup(self, workgroup, entityID):
    driver = self.driver
    self.search_for_sp(entityID)

    update_link = driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]")
    self.progress('update_link')
    self.progress(self.pp(update_link))

    if (not update_link):
      self.exit_with_exception("could not find update icon using css selector")

    update_link.click()
    WebDriverWait(driver, 30).until(readystate_complete)

    # Add the workgroup
    self.add_workgroup_on_edit_form(workgroup)

    driver.find_element_by_link_text("Save Changes").click()
    WebDriverWait(driver, 30).until(readystate_complete)
    time.sleep(2);

    try:
      owning_workgroups = driver.find_element_by_id('owning_workgroups')
    except NoSuchElementException, e:
      return None

    return owning_workgroups.get_attribute('innerHTML')

  def read_metadata_file(self, filename):
    fh = open(filename, 'r')
    string = fh.read()
    fh.close()
    return string

  @staticmethod
  def get_entityID_from_metadata(metadata):
    rx = re.compile(r'entityID\s*=\s*"([^"]*)"')
    match = re.search(rx, metadata)
    if (match):
      return match.group(1)
    else:
      return None

  @staticmethod
  def make_subject(domain):
    subject_skel='/C=US/ST=California/O=Stanford University/localityName=Palo Alto/commonName=DOMAIN/organizationalUnitName=Administrative Systems/emailAddress=as-acs-linux-team@lists.stanford.edu'

    return subject_skel.replace('DOMAIN', domain)

  @staticmethod
  def create_cert(subject_name, days):
    keyfile = tempfile.NamedTemporaryFile()
    pemfile = tempfile.NamedTemporaryFile()

    subject = SPDBTesting.make_subject(subject_name)
    command = ['openssl',
               'req',
               '-x509',
               '-nodes',
               '-newkey', 'rsa:2048',
               '-keyout', keyfile.name,
               '-out',    pemfile.name,
               '-days',   days,
               '-subj',   subject
               ]
    #print command
    call(command)

    # Get the pem file
    fh = open(pemfile.name, 'r')
    pem_string = fh.read()
    fh.close()

    # Remove the '-----BEGIN CERTIFICATE-----' and
    # '-----END CERTIFICATE-----' strings.
    pem_string = string.replace(pem_string, '-----BEGIN CERTIFICATE-----', '')
    pem_string = string.replace(pem_string, '-----END CERTIFICATE-----', '')
    pem_string = string.strip(pem_string)

    # We can now delete the temporary files.
    keyfile.close()
    pemfile.close()

    return pem_string

  @staticmethod
  def generate_random_domain():
    hostname = ''.join(random.choice(string.ascii_letters) for x in range(8))
    return hostname + '.stanford.edu'

  @staticmethod
  def generate_metadata(domain, cert1, cert2 = None, use_signature = False):

    cert_subject = SPDBTesting.make_subject(domain)

    if (cert2 is None):
      metadata_template = 'meta1.xml'
    else:
      metadata_template = 'meta2.xml'

    fh = open(metadata_template, 'r')
    template = fh.read()

    domain_id = domain.replace('/', '')

    # Note that RANDOM1 is the ID attribute hile RANDOM2 is the entityID
    # attribute.
    template = re.sub(r'RANDOM1',  domain_id,    template)
    template = re.sub(r'RANDOM2',  domain,       template)
    template = re.sub(r'CERT1',    cert1,        template)
    template = re.sub(r'SUBJECT1', cert_subject, template)

    if (cert2 is not None):
      template = re.sub(r'CERT2',    cert2,        template)
      template = re.sub(r'SUBJECT1', cert_subject, template)

    # Add signature (if use_signature is true)
    if (use_signature):
      with open('signature.xml','r') as f:
        signature_string = f.read()
    else:
      signature_string = ''

    template = re.sub(r'PUT_SIGNATURE_HERE', signature_string, template)

    return template

  @staticmethod
  def parse_args():
    parser = argparse.ArgumentParser(description='Run Selenium tests')
    parser.add_argument('--environment', help='SPDB environment (PROD, STAGE, UAT, DEV)',
                        )
    parser.add_argument('--browser', help='browser model',
                        choices=['chrome', 'firefox', 'phantomjs'])
    parser.add_argument('--entityID', help='explicitly specify the entityID'
                        )
    parser.add_argument('--generate-metadata', action='store_true',
                        help='generate metadata files only'
                        )
    parser.add_argument('--use-metadata-files', action='store_true',
                        help='use existing metadata files'
                        )

    parser.add_argument('--tests', help='comma-separated list of tests to run (or ALL)'
                        )

    parser.add_argument('--verbose', action='store_true',
                        help='be verbose'
                        )
    return parser.parse_args()

  @staticmethod
  def generate_metadata_file(filename, domain, cert1, cert2 = None, use_signature = False):
    template = SPDBTesting.generate_metadata(domain, cert1, cert2, use_signature)
    with open(filename, "w") as text_file:

      if (filename == 'test_validUntil_expired.xml'):
        _template_ = template.replace('2050-01-01', '2000-01-01')
      elif (filename == 'test_unprefixed_acs_metadata.xml'):
        # _template_ = template.replace('md:AssertionConsumerService', 'AssertionConsumerService')
        _template_ = template.replace('md:', '').replace(':md', '')
      else:
        _template_ = template

      text_file.write(_template_)

    print "generated file " + filename
    return


  # Generates these metadata files in the current directory:
  # * test_single_valid_cert_metadata.xml
  # * test_single_expired_cert_metadata.xml
  # * test_two_expired_cert_metadata.xml
  # * test_one_expired_one_valid_cert_metadata.xml
  #
  # We do this for those platforms that do not have the openssl executable
  # installed.
  #
  # Note that ALL the metadata files have the SAME entityID.
  @staticmethod
  def generate_metadata_files(domain):
    valid_cert1   = SPDBTesting.create_cert(domain, '3650')
    valid_cert2   = SPDBTesting.create_cert(domain, '3650')
    expired_cert1 = SPDBTesting.create_cert(domain, '-1')
    expired_cert2 = SPDBTesting.create_cert(domain, '-1')
    non_self_signed_cert1 =  open('incommon-usertrust-2024.pem', 'r').read()

    filename_to_template = {}

    SPDBTesting.generate_metadata_file('test_single_valid_cert_metadata.xml', domain, valid_cert1)

    SPDBTesting.generate_metadata_file('test_validUntil_expired.xml', domain, valid_cert1)

    SPDBTesting.generate_metadata_file('test_incommon_entity_id.xml', 'northwestern.illiad.oclc.org/shibboleth', valid_cert1)

    SPDBTesting.generate_metadata_file('test_single_expired_cert_metadata.xml', domain, expired_cert1)

    SPDBTesting.generate_metadata_file('test_two_expired_cert_metadata.xml', domain, expired_cert1, expired_cert2)

    SPDBTesting.generate_metadata_file('test_one_expired_one_valid_cert_metadata.xml', domain, expired_cert1, valid_cert1)

    SPDBTesting.generate_metadata_file('test_unprefixed_acs_metadata.xml', domain, expired_cert1, valid_cert1)

    SPDBTesting.generate_metadata_file('test_non_self_signed_cert.xml', domain, non_self_signed_cert1)

    SPDBTesting.generate_metadata_file('test_contains_signature.xml',
                                       domain,
                                       non_self_signed_cert1,
                                       use_signature = True)

    # The bad metadata file
    fp = open('test_single_invalid_metadata.xml', 'w')
    fp.write('bad metadata')
    fp.close()


  @staticmethod
  def assertRegexpMatches(ut, text, expected_regexp, msg=None):
    try:
      unittest.TestCase.assertRegexpMatches(ut, text, expected_regexp, msg)
    except Exception, e:
      # print text
      raise ut.failureException(msg or "Regexp \"" + expected_regexp + "\" didn't match")

  # Read the metadata from the file <filename> and return any text from
  # the red error box (might return None if no error text).
  def testMetadataWrapper(self, filename):
    # Get the metadata.
    metadata = self.read_metadata_file(filename)
    # self.entityID = self.SPDB.get_entityID_from_metadata(metadata)

    # Test it:
    self.test_metadata(metadata)

    return self.get_red_error_text()

  # Find the spconfig matching entityID and delete it.
  # Returns true if actually deleted, false otherwise.
  def deleteAWSConfig(self, browser):

    result = self.search_for_aws(self.awsaccount)

    if (result == 0):
      print "\tNo AWS record found. Skipping..."
      return

    driver = self.driver

    if (browser == 'phantomjs'):
      #
      # added new hidden link element without confirmation
      #
      element = driver.find_element_by_id ("delete_hidden")
      if (not element):
        self.exit_with_exception("[phantomjs] cannot find delete_hidden element")
      element.click()
    else:
      #
      # "visual" browsers use the alert pop-up
      #
      driver.find_element_by_css_selector("img[alt=\"Deleteicon enabled\"]").click()
      time.sleep(1)
      Alert(driver).accept()


class SPDBUnitTest(unittest.TestCase):

  ENVIRONMENT        = 'DEV'
  BROWSER            = 'chrome'
  GENERATE_METADATA  = False
  USE_METADATA_FILES = False
  SPDB               = None
  USERNAME           = 'unknown'
  TESTS              = 'ALL'
  VERBOSE            = False

  # Workgroup settings
  WG1 = 'as-fss:tech'
  WG2 = 'as-fss:dmrteam'

  # WG_USER1 belongs to workgroup WG1. Note that WG_USER1 is an admin, so
  # be sure to run some of the tests as a non-admin.
  WG_USER1 = 'cheroux'

  # WG_USER2 does NOT belong to workgroup WG1. WG_USER2 is NOT an admin.
  WG_USER2 = 'jheroux'

  # WG_USER3 belongs to workgroup WG1.
  WG_USER3 = 'rraman'

  # ADMIN users
  WG_ADMIN   = 'cheroux'

  # REGULAR user
  WG_REGULAR = 'jheroux'

  @classmethod
  def setUpClass(cls):
    # Get comand line args

    # if (cls.USE_METADATA_FILES):
    #   print "using existing metadata files..."
    # else:
    #   # print "generating metadata files..."
    #   # domain = SPDBTesting.generate_random_domain()
    #   # print "generating metadata files for domain '" + domain + "'"
    #   # SPDBTesting.generate_metadata_files(domain)
    #   # print "Your entityID is " + domain

    print "running in environment " + cls.ENVIRONMENT
    cls.SPDB = SPDBTesting(environment = cls.ENVIRONMENT,
                           browser     = cls.BROWSER,
                           verbose     = cls.VERBOSE,
    )

  def setUp(self):
    pass

  def progress(self, message):
    if (self.VERBOSE):
      print('progress: ' + message)

  # Call this method before running any workgroup test.
  def set_username(self, user_name):
    self.SPDB.reset_username(user_name)
    return

  def get_page_source(self):
    return self.SPDB.driver.page_source.encode('utf-8').strip()

  #
  # test all 4 button elements
  #
  def test_001_general(self):
    print "." + inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    self.SPDB.go_home()
    driver = self.SPDB.driver

    print "\tSearching for Add New..."
    self.assertRegexpMatches(driver.page_source, 'Add New')
    print "\tSearching for List My..."
    self.assertRegexpMatches(driver.page_source, 'List My')
    print "\tSearching for Search Accounts..."
    self.assertRegexpMatches(driver.page_source, 'Search Accounts')
    print "\tSearching for List All..."
    self.assertRegexpMatches(driver.page_source, 'List All')

  def test_002_check_menu_buttons(self):
    print "." + inspect.stack()[0][3]

    print "\tGoing to Add New AWS Account..."
    source = self.SPDB.go_to_menu_option('button_spconfig_new')
    self.assertRegexpMatches(source, 'New AWS Account')

    print "\tGoing to List My AWS Accounts..."
    source = self.SPDB.go_to_menu_option('button_spconfig_all')
    self.assertRegexpMatches(source, 'My AWS Accounts')

    print "\tGoing to Search AWS Accounts..."
    source = self.SPDB.go_to_menu_option('button_spconfig_search')
    self.assertRegexpMatches(source, 'Account Number')

    print "\tGoing to List My AWS Accounts..."
    source = self.SPDB.go_to_menu_option('button_awsaccounts_all')
    self.assertRegexpMatches(source, 'My AWS Accounts')

  def test_003_create_new_aws_account(self):
    print "." + inspect.stack()[0][3]

    driver = self.SPDB.driver

    print "\tDeleting AWS Config for " + self.SPDB.awsaccount + " if exists..."
    self.SPDB.deleteAWSConfig(self.BROWSER)

    print "\tLaunching Add New AWS Account..."
    source = self.SPDB.go_to_menu_option('button_spconfig_new')
    element = self.SPDB.get_element_by_id("awsaccount_number")

    print "\tEntering Account ID " + self.SPDB.awsaccount + "..."
    self.SPDB.send_keys_slow(element, self.SPDB.awsaccount)

    element = self.SPDB.get_element_by_id("awsaccount_nickname")
    self.SPDB.send_keys_slow(element, "nickname")

    print "\tAdding Workgroup..."
    self.SPDB.add_workgroup_on_edit_form(self.WG1 + '|0')

    print "\tAdding Notes..."
    element = self.SPDB.get_element_by_id("awsaccount_notes")
    self.SPDB.send_keys_slow(element, "Notes for selenium testing")

    print "\tSubmitting Form..."
    driver.find_element_by_partial_link_text('Create Account').click()
    WebDriverWait(driver, 30).until(readystate_complete)

  def test_004_edit_new_aws_account(self):
    print "." + inspect.stack()[0][3]

    self.SPDB.go_home()
    driver = self.SPDB.driver

    print "\tEditing AWS Account " + self.SPDB.awsaccount + "..."

    element = self.SPDB.get_element_by_id("search")
    self.SPDB.send_keys_slow(element, self.SPDB.awsaccount)

    print "\tSearching for account " + self.SPDB.awsaccount + "..."
    driver.find_element_by_id('button_spconfig_search').click()
    WebDriverWait(driver, 30).until(readystate_complete)

    #
    # Hit edit button
    #
    print "\tSelecting update icon..."
    driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    print "\tAppending \"update test\" to awsaccount_notes..."
    element = self.SPDB.get_element_by_id("awsaccount_notes")
    self.SPDB.send_keys_slow(element, "Notes for selenium testing - update test")

    print "\tSaving Form..."
    driver.find_element_by_partial_link_text('Save Changes').click()
    WebDriverWait(driver, 30).until(readystate_complete)

    text = self.SPDB.get_message_text()
    print "\tUpdate text: " + text.rstrip().replace('\n', '') + "..."

  def test_005_new_aws_account_validate_form(self):
    print "." + inspect.stack()[0][3]

    driver = self.SPDB.driver

    print "\tDeleting AWS Config for " + self.SPDB.awsaccount + " if exists..."
    self.SPDB.deleteAWSConfig(self.BROWSER)

    print "\tLaunching Add New AWS Account Validate Form..."
    source = self.SPDB.go_to_menu_option('button_spconfig_new')

    print "\tEntering Account ID " + self.SPDB.awsaccount + "..."
    element = self.SPDB.get_element_by_id("awsaccount_number")
    self.SPDB.send_keys_slow(element, self.SPDB.awsaccount)

    print "\tAdding Workgroup..."
    self.SPDB.add_workgroup_on_edit_form(self.WG1 + '|0')

    print "\tAdding Notes..."
    element = self.SPDB.get_element_by_id("awsaccount_notes")
    self.SPDB.send_keys_slow(element, "Notes for selenium testing")

    print "\tSubmitting Form without entering nickname..."
    driver.find_element_by_partial_link_text('Create Account').click()
    WebDriverWait(driver, 30).until(readystate_complete)

    error_text = self.SPDB.get_red_error_text()
    self.assertIn('An AWS account nickname must be provided', error_text,
                  'An AWS account nickname must be provided')
    print "\tTest passed"

    print "\tEntering nickname F^###S**"
    element = self.SPDB.get_element_by_id("awsaccount_nickname")
    self.SPDB.send_keys_slow(element, "F^###S**")

    print "\tSubmitting Form with funky nickname..."
    driver.find_element_by_partial_link_text('Create Account').click()
    WebDriverWait(driver, 30).until(readystate_complete)

    error_text = self.SPDB.get_red_error_text()
    self.assertIn('A nickname can only contain letters', error_text,
                  'A nickname can only contain letters, numbers, dashes, and spaces, and must be between 3 and 64 characters long')
    print "\tTest passed"

    driver.find_element_by_id("awsaccount_nickname").clear()
    element = self.SPDB.get_element_by_id("awsaccount_nickname")
    self.SPDB.send_keys_slow(element, "nickname")

    driver.find_element_by_id("awsaccount_number").clear()
    element = self.SPDB.get_element_by_id("awsaccount_number")
    self.SPDB.send_keys_slow(element, "000000000000")

    print "\tSubmitting Form with account 000000000000..."
    driver.find_element_by_partial_link_text('Create Account').click()
    WebDriverWait(driver, 30).until(readystate_complete)

    error_text = self.SPDB.get_red_error_text()
    self.assertIn('is not allowed to be managed by this application', error_text,
                  'The account number \'000000000000\' is not allowed to be managed by this application')
    print "\tTest passed"

def suite():
  all_methods = inspect.getmembers(SPDBUnitTest, predicate=inspect.ismethod)
  test_methods = filter(lambda x: re.match(r'^test_\d\d\d', x[0]), all_methods)
  test_method_names = map(lambda x: x[0], test_methods)

  tests = []
  if (SPDBUnitTest.TESTS == 'ALL'):
    tests = sorted(test_method_names)
  else:
    matching_tests = []
    # Split tests into individuals.
    test_numbers = SPDBUnitTest.TESTS.split(',')

    # Add the cleanup test
    test_numbers.append('999')

    # Find the tests corresponding to these.
    for test_number in test_numbers:
      if (not re.match(r'^\d\d\d$', test_number)):
        raise Exception("'" + test_number + "' is not three digits.")

      test_matching = filter(lambda x: 'test_' + test_number in x, test_method_names)
      if (test_matching):
        matching_tests.append(test_matching[0])

    tests = sorted(matching_tests)


  tests = [
    'test_001_general',
    'test_002_check_menu_buttons',
    'test_003_create_new_aws_account',
    'test_004_edit_new_aws_account',
    'test_005_new_aws_account_validate_form',
  ]

  print "Will run these tests:"
  for test in tests:
    print ' - ' + test

  return unittest.TestSuite(map(SPDBUnitTest, tests))


if __name__ == "__main__":
  args = SPDBTesting.parse_args()

  if (args.generate_metadata):
    SPDBUnitTest.GENERATE_METADATA = False
    sys.argv.pop()

  if (args.use_metadata_files):
    SPDBUnitTest.USE_METADATA_FILES = False
    sys.argv.pop()

  if (args.browser):
    SPDBUnitTest.BROWSER = args.browser
    sys.argv.pop()

  if (args.environment):
    SPDBUnitTest.ENVIRONMENT = args.environment
    sys.argv.pop()

  if (args.tests):
    SPDBUnitTest.TESTS = args.tests
  else:
    SPDBUnitTest.TESTS = 'ALL'

  if (args.verbose):
    SPDBUnitTest.VERBOSE = True
    sys.argv.pop()

  if (args.generate_metadata):
    domain = SPDBTesting.generate_random_domain()
    print "generating metadata files for domain '" + domain + "'"
    SPDBTesting.generate_metadata_files(domain)
    print "Your entityID is " + domain
    sys.exit(0)
  else:
    print "About to run TestRunner"
    #unittest.main(exit=False)
    #
    unittest.TextTestRunner(verbosity=1).run(suite())
    #

  if (args.browser == "phantomjs"):
    #
    # PhantomJS may not exit properly
    #
    pid = os.getpid()
    os.system("pkill -KILL -P " + str(pid))
